@@ -37,6 +37,7 @@ static DBusConnection *connection;
 static GNode *element_root = NULL;
 static GSList *driver_list = NULL;
 static gchar *device_filter = NULL;
+static gchar *nodevice_filter = NULL;
 
 static gboolean started = FALSE;
 
@@ -272,10 +273,7 @@ struct connman_service *__connman_element_get_service(struct connman_element *el
 	switch (type) {
 	case CONNMAN_DEVICE_TYPE_UNKNOWN:
 	case CONNMAN_DEVICE_TYPE_VENDOR:
-	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
 	case CONNMAN_DEVICE_TYPE_GPS:
-	case CONNMAN_DEVICE_TYPE_MBM:
-	case CONNMAN_DEVICE_TYPE_HSO:
 	case CONNMAN_DEVICE_TYPE_NOZOMI:
 	case CONNMAN_DEVICE_TYPE_HUAWEI:
 	case CONNMAN_DEVICE_TYPE_NOVATEL:
@@ -285,6 +283,9 @@ struct connman_service *__connman_element_get_service(struct connman_element *el
 		break;
 	case CONNMAN_DEVICE_TYPE_WIFI:
 	case CONNMAN_DEVICE_TYPE_WIMAX:
+	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
+	case CONNMAN_DEVICE_TYPE_MBM:
+	case CONNMAN_DEVICE_TYPE_HSO:
 		network = __connman_element_get_network(element);
 		if (network == NULL)
 			return NULL;
@@ -732,6 +733,7 @@ static int set_static_property(struct connman_element *element,
 	case DBUS_TYPE_STRING:
 		property->value = g_strdup(*((const char **) value));
 		break;
+	case DBUS_TYPE_BOOLEAN:
 	case DBUS_TYPE_BYTE:
 		property->value = g_try_malloc(1);
 		if (property->value != NULL)
@@ -910,6 +912,7 @@ static gboolean get_static_property(struct connman_element *element,
 			*((char **) value) = property->value;
 			found = TRUE;
 			break;
+		case DBUS_TYPE_BOOLEAN:
 		case DBUS_TYPE_BYTE:
 			memcpy(value, property->value, 1);
 			found = TRUE;
@@ -1003,6 +1006,38 @@ const char *connman_element_get_string(struct connman_element *element,
 	return value;
 }
 
+/**
+ * connman_element_set_bool:
+ * @element: element structure
+ * @key: unique identifier
+ * @value: boolean value
+ *
+ * Set boolean value for specific key
+ */
+int connman_element_set_bool(struct connman_element *element,
+					const char *key, connman_bool_t value)
+{
+	return set_static_property(element, key, DBUS_TYPE_BOOLEAN, &value);
+}
+
+/**
+ * connman_element_get_bool:
+ * @element: element structure
+ * @key: unique identifier
+ *
+ * Get boolean value for specific key
+ */
+connman_bool_t connman_element_get_bool(struct connman_element *element,
+							const char *key)
+{
+	connman_bool_t value;
+
+	if (get_static_property(element, key, &value) == FALSE)
+		return FALSE;
+
+	return value;
+}
+
 /**
  * connman_element_set_uint8:
  * @element: element structure
@@ -1014,7 +1049,7 @@ const char *connman_element_get_string(struct connman_element *element,
 int connman_element_set_uint8(struct connman_element *element,
 					const char *key, connman_uint8_t value)
 {
-        return set_static_property(element, key, DBUS_TYPE_BYTE, &value);
+	return set_static_property(element, key, DBUS_TYPE_BYTE, &value);
 }
 
 /**
@@ -1030,7 +1065,7 @@ connman_uint8_t connman_element_get_uint8(struct connman_element *element,
 	connman_uint8_t value;
 
 	if (get_static_property(element, key, &value) == FALSE)
-                return 0;
+		return 0;
 
 	return value;
 }
@@ -1337,6 +1372,28 @@ static void register_element(gpointer data, gpointer user_data)
 	probe_element(element);
 }
 
+gboolean __connman_element_device_isfiltered(const char *devname)
+{
+	if (device_filter == NULL)
+		goto nodevice;
+
+	if (g_pattern_match_simple(device_filter, devname) == FALSE) {
+		DBG("ignoring device %s (match)", devname);
+		return TRUE;
+	}
+
+nodevice:
+	if (nodevice_filter == NULL)
+		return FALSE;
+
+	if (g_pattern_match_simple(nodevice_filter, devname) == TRUE) {
+		DBG("ignoring device %s (no match)", devname);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 /**
  * connman_element_register:
  * @element: the element to register
@@ -1355,15 +1412,13 @@ int connman_element_register(struct connman_element *element,
 	if (element->devname == NULL)
 		element->devname = g_strdup(element->name);
 
-	if (device_filter && element->type == CONNMAN_ELEMENT_TYPE_DEVICE) {
-		if (g_pattern_match_simple(device_filter,
-						element->devname) == FALSE) {
-			DBG("ignoring %s [%s] device", element->name,
-							element->devname);
-			return -EPERM;
-		}
-	}
+        if (element->type != CONNMAN_ELEMENT_TYPE_DEVICE)
+		goto setup;
 
+	if (__connman_element_device_isfiltered(element->devname) == TRUE)
+		return -EPERM;
+
+setup:
 	if (connman_element_ref(element) == NULL)
 		return -EINVAL;
 
@@ -1535,18 +1590,18 @@ void connman_element_set_error(struct connman_element *element,
 	__connman_service_indicate_error(service, convert_error(error));
 }
 
-int __connman_element_init(DBusConnection *conn, const char *device,
-							const char *nodevice)
+int __connman_element_init(const char *device, const char *nodevice)
 {
 	struct connman_element *element;
 
-	DBG("conn %p", conn);
+	DBG("");
 
-	connection = dbus_connection_ref(conn);
+	connection = connman_dbus_get_connection();
 	if (connection == NULL)
 		return -EIO;
 
 	device_filter = g_strdup(device);
+	nodevice_filter = g_strdup(nodevice);
 
 	element = connman_element_create("root");
 
@@ -1596,7 +1651,9 @@ void __connman_element_start(void)
 
 	__connman_connection_init();
 	__connman_ipv4_init();
-	__connman_rfkill_init();
+
+	if (__connman_rfkill_init() < 0)
+		__connman_udev_enable_rfkill_processing();
 }
 
 void __connman_element_stop(void)
@@ -1604,6 +1661,7 @@ void __connman_element_stop(void)
 	DBG("");
 
 	__connman_rfkill_cleanup();
+
 	__connman_ipv4_cleanup();
 	__connman_connection_cleanup();
 }
@@ -1656,7 +1714,11 @@ void __connman_element_cleanup(void)
 	g_node_destroy(element_root);
 	element_root = NULL;
 
+	g_free(nodevice_filter);
 	g_free(device_filter);
 
+	if (connection == NULL)
+		return;
+
 	dbus_connection_unref(connection);
 }
